//------------------------------------------------------------------------------
//             Copyright (c) 2002-2003, Tech-Know-Ware Ltd.
//------------------------------------------------------------------------------


// File name:   lumas-lib-test.c
//------------------------------------------------------------------------------
// Description: This file has been produced by Tech-Know-Ware to test and 
//				demonstrate the use of the Lumas Library.
//
// Notice:      TERMS & CONDITIONS
//
//              Your use of this file implies agreement to the Terms & 
//              Conditions as presented in this notice.
//
//              Subject to the terms and conditions of this notice, 
//              TECH-KNOW-WARE LTD hereby grants YOU (the person accepting this 
//              agreement) a non-exclusive right to use this file.
//
//              THIS FILE AND THE SOFTWARE CONTAINED HEREIN IS PROVIDED 'AS IS' 
//              AND COMES WITH NO WARRENTY OF ANY KIND.  
//
//              TECH-KNOW-WARE LTD SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED 
//              BY ANYONE OR ANYTHING DUE TO THE USE OF THIS FILE HOWEVER THEY 
//              MAY BE CAUSED.
//
//              This file, or any derivative of it, is provided on the condition
//              that YOU may not supply it to a third party outside of the 
//              organisation to which you are affiliated in a form that 
//              allows the third party to compile or link the software into 
//              programs of their own making.  Other than the previous 
//              condition, this file, or any derivative of it, may be used for 
//              any purpose, including commercial purposes. 
//
//              Users are permitted to modify the contents of this file, as 
//              long this notice and the copyright notice remains and the 
//              modified file is given a name that can not be confused with 
//              the files provided by Tech-Know-Ware Ltd.
//
//              Users are permitted to extract elements of this file into
//              other files, subject to this notice and the copyright notice 
//              also being copied into the target files.
//
//              For more information on Lumas, visit:
//
//                  http://www.tech-know-ware.com/lumas
//
//              If you find a bug in this software, please report it to:
//
//                  lumas-lib-bugs@tech-know-ware.com
//
//              Copyright (c) 2002, 2003 Tech-Know-Ware Ltd.
//
//              End of Notice.
//
// Module Tag:  lml - Lumas Library.
// Version:     V.96
//
// Entry Points: main


#include <stdlib.h>
#include <string.h>

#include "lumas-lib.h"

extern struct slml_tag_info mlml_lumas_test[];


int f_lumas_cpp_test( char *ap_in_file, char *ap_out_file );

static double p_zero()
{
	return 0.0;
}


void f_lumas_c_test( char *ap_in_file, char *ap_out_file )
{
    FILE *lh_fin;
	int l_i;
	int l_c;
    slml_parse_ctrl l_parse_ctrl;


    // Use mode "rb" so that we can read UTF-8
    if( (lh_fin = fopen( ap_in_file, "rb" )) != NULL )
        {
        FILE *lh_debug_file = NULL;
        int l_value;
		unsigned char lc_data[256];
		char lc_message[1024];
		int l_message_length;

        if( ap_out_file != NULL )
            lh_debug_file = fopen( ap_out_file, "wt" );
        else
            lh_debug_file = stdout;

		// Read in test message
		for( l_i=0; (l_c = fgetc( lh_fin )) != EOF && l_i < sizeof( lc_message ); l_i++ )
			lc_message[l_i] = l_c;


		if( (l_message_length = flml_is_message_complete( lc_message, l_i )) != -1 )
			fprintf( lh_debug_file, "Message is complete, length: %d\n\n", l_message_length );
		else
			fprintf( lh_debug_file, "Message is incomeplete\n\n" );

		if( flml_is_well_formed( lc_message, l_message_length ) )
			fprintf( lh_debug_file, "Message is well formed\n\n" );
		else
			fprintf( lh_debug_file, "Message is NOT well formed\n\n" );

		flml_set_tag_info_root( &l_parse_ctrl, mlml_lumas_test );

        if( flml_parse_message( &l_parse_ctrl, lc_message, l_message_length ) )
			fprintf( lh_debug_file, "Message parsed OK\n\n" );
		else
			fprintf( lh_debug_file, "Message NOT parsed\n\n" );


        fprintf( lh_debug_file, "Results of tree parse operation\n"
                                "-------------------------------\n" );

        flml_debug_display_parse_info( lh_debug_file, &l_parse_ctrl );


        fprintf( lh_debug_file, "\n\n" );


        fprintf( lh_debug_file, "Results of extracting parameters\n"
                                "--------------------------------\n" );


		// Set location to first untagged struct
		flml_set_location( &l_parse_ctrl, "=", 0, NULL );


        {
        char lc_host_name[128];


		// Read first untagged parameter
        if( flml_read_ascii( &l_parse_ctrl,
								"0=0",
								0, 
								lc_host_name,
								sizeof( lc_host_name ) ) )
            fprintf( lh_debug_file, "Untagged host-name (myco.com) is: %s\n", lc_host_name );
        else
            fprintf( lh_debug_file, "Untagged host-name is: NOT FOUND\n" );


		// Read second instance of second untagged parameter
        if( flml_read_integer( &l_parse_ctrl,
								"0=1",
								1, 
								&l_value ) )
            fprintf( lh_debug_file, "Second untagged port parameter (40000) is: %d\n", l_value );
        else
            fprintf( lh_debug_file, "Second untagged port parameter is NOT FOUND\n" );

		// Read second instance of second untagged parameter using instance in param spec
        if( flml_read_integer( &l_parse_ctrl,
								"0=1,1",
								-1, 
								&l_value ) )
            fprintf( lh_debug_file, "Second untagged port parameter [0=1,1] (40000) is: %d\n", l_value );
        else
            fprintf( lh_debug_file, "Second untagged port parameter is NOT FOUND\n" );
        }


		// Read the value of the union tagged by T
		if( flml_read_union( &l_parse_ctrl,  
							"T",
							0, lc_data, sizeof(lc_data) ) )
			fprintf( lh_debug_file, "Selected UNION in 'T' (TR) is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "Selected UNION in 'T' (TR) is NOT FOUND\n" );

		{
		// Enumerate the value of the union tagged by T
		slml_tag_translation lc_tag_list[] = {
			"TRj", 0,
			"TR", 1,
			"T", 2,
			"", 3,
			NULL, 0 };

		fprintf( lh_debug_file, "Selected UNION in 'T' (1) is: %d\n",
						flml_read_enumerated_union( &l_parse_ctrl, 
                                                    "T",
                                                    0, 
													lc_tag_list ) );
		}


        {
        char lc_line_name[128];

		// Move to the struct tagged by T and then TR
        slml_location l_old_location;

		flml_set_location( &l_parse_ctrl, "T=TR", 0, &l_old_location );

		// Read the first untagged integer
        if( flml_read_integer( &l_parse_ctrl,
								"0",
								0, 
								&l_value ) )
            fprintf( lh_debug_file, "Transaction Id (543345): %d\n", l_value );
        else
            fprintf( lh_debug_file, "Transaction Id: NOT FOUND\n" );


		// Read the first untagged integer in the A struct
        if( flml_read_integer( &l_parse_ctrl,
								"A=0",
								0, 
								&l_value ) )
            fprintf( lh_debug_file, "Context Id (325465): %d\n", l_value );
        else
            fprintf( lh_debug_file, "Context Id: NOT FOUND\n" );


		// Read the value specified in the untagged union in A  cmd
        if( flml_read_union( &l_parse_ctrl,  
                            "A=cmd=0",
                            0, lc_data, sizeof(lc_data) ) )
			fprintf( lh_debug_file, "Selected 'cmd' (add) is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "Selected 'cmd' (add) is NOT FOUND\n", lc_data );


		// Read the untagged ascii string in A cmd
        if( flml_read_ascii( &l_parse_ctrl,
								"A=cmd=1",
								0, 
								lc_line_name,
								sizeof( lc_line_name ) ) )
            fprintf( lh_debug_file, "Line name (Line 'one') is: %s\n", lc_line_name );
        else
            fprintf( lh_debug_file, "Line name is: NOT FOUND\n" );


		// Try to read line name into something just the right size
        if( flml_read_ascii( &l_parse_ctrl,
								"A=cmd=1",
								0, 
								lc_line_name,
								strlen( lc_line_name ) + 1 ) )
            fprintf( lh_debug_file, "Re-read Line name (Line 'one') is: %s\n", lc_line_name );
        else
            fprintf( lh_debug_file, "Re-read Line name is: TOO SMALL\n" );


		// Try to read line name into something too small
        if( flml_read_ascii( &l_parse_ctrl,
								"A=cmd=1",
								0, 
								lc_line_name,
								strlen( lc_line_name ) ) )
            fprintf( lh_debug_file, "Re-read Line name (incorrect) is: %s\n", lc_line_name );
        else
            fprintf( lh_debug_file, "Re-read Line name is: TOO SMALL (CORRECT)\n" );


		// Move from the root of the message tree to the second instance of cmd in T=TR=A
        flml_set_location( &l_parse_ctrl, "A=cmd", 1, NULL ); 
                                                   

		// Read the value of the first untagged union
        if( flml_read_union( &l_parse_ctrl, 
                            "0",
                            0, lc_data, sizeof(lc_data) ) )
			fprintf( lh_debug_file, "Second Selected 'cmd' (sc) is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "Second Selected 'cmd' (sc) is NOT FOUND\n" );


        if( flml_read_ascii( &l_parse_ctrl,
								"=T=TR=A=cmd,1=1",
								0, 
								lc_line_name,
								sizeof( lc_line_name ) ) )
            fprintf( lh_debug_file, "2nd Line name [=T=TR=A=cmd,1=1] (Line 2) is: %s\n", lc_line_name );
        else
            fprintf( lh_debug_file, "2nd Line name [=T=TR=A=cmd,1=1] is: NOT FOUND\n" );


		// Read the ascii string of the second untagged value
        if( flml_read_ascii( &l_parse_ctrl,
								"1",
								0, 
								lc_line_name,
								sizeof( lc_line_name ) ) )
            fprintf( lh_debug_file, "2nd Line name [Relative addr] (Line 2) is: %s\n", lc_line_name );
        else
            fprintf( lh_debug_file, "2nd Line name [Relative addr] is: NOT FOUND\n" );


		// Return to the previous location in the message tree
		flml_restore_location( &l_parse_ctrl, l_old_location );
        }

		{
		float l_float;

		if( flml_read_float( &l_parse_ctrl, "float-nan", 0, &l_float ) )
			fprintf( lh_debug_file, "float-nan (NaN) is: %f\n", l_float );
		else
			fprintf( lh_debug_file, "float-nan is: NOT FOUND\n" );

		if( flml_read_float( &l_parse_ctrl, "float-inf", 0, &l_float ) )
			fprintf( lh_debug_file, "float-inf (INF) is: %f\n", l_float );
		else
			fprintf( lh_debug_file, "float-inf is: NOT FOUND\n" );

		if( flml_read_float( &l_parse_ctrl, "float--inf", 0, &l_float ) )
			fprintf( lh_debug_file, "float--inf (-INF) is: %f\n", l_float );
		else
			fprintf( lh_debug_file, "float--inf is: NOT FOUND\n" );

		if( flml_read_float( &l_parse_ctrl, "float", 0, &l_float ) )
			fprintf( lh_debug_file, "float (12.43e-32) is: %g\n", l_float );
		else
			fprintf( lh_debug_file, "float is: NOT FOUND\n" );
		}


        if( flml_read_union( &l_parse_ctrl, 
                            "T",
                            1, lc_data, sizeof(lc_data) ) )
			fprintf( lh_debug_file, "2nd Selected UNION in 'T' (TRj) is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "2nd Selected UNION in 'T' is: NOT FOUND\n" );



        if( flml_read_union( &l_parse_ctrl, 
                            "T",
                            2, lc_data, sizeof(lc_data) ) )
			fprintf( lh_debug_file, "3rd Selected UNION in 'T' (Test) is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "3rd Selected UNION in 'T' is: NOT FOUND\n" );



		flml_set_location( &l_parse_ctrl, "T", 2, NULL );


        if( flml_read_union( &l_parse_ctrl, 
                            "Test=0",
                            0, lc_data, sizeof(lc_data) ) )
			fprintf( lh_debug_file, "First untagged union of Test (*) is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "First untagged union of Test is: NOT FOUND\n" );

		
		{
		// Enumerate the value of the union tagged by T
		slml_tag_translation lc_tag_list[] = {
			"", 0,
			"*", 1,
			NULL, 0 };

		fprintf( lh_debug_file, "Selected UNION in 'Test=0' (1) is: %d\n",
						flml_read_enumerated_union( &l_parse_ctrl, 
                                                    "Test=0",
                                                    0, 
													lc_tag_list ) );
		}

		
        if( flml_read_union( &l_parse_ctrl, 
                            "Test=2volume",
                            0, lc_data, sizeof(lc_data) ) )
			fprintf( lh_debug_file, "Volume (14) is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "Volume is: NOT FOUND\n" );

		
        if( flml_read_union( &l_parse_ctrl, 
                            "Test=2addr",
                            0, lc_data, sizeof(lc_data) ) )
			fprintf( lh_debug_file, "Address type (IPv4) is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "Address type is: NOT FOUND\n" );


		{
		// Read the IPv4 address value at Test = addr = IPv4
		unsigned char lc_addr[4];
        if( flml_read_ipv4( &l_parse_ctrl, 
                            "Test=2addr=IPv4",
                            0, lc_addr ) )
			fprintf( lh_debug_file, "Address (212.10.18.4) is: %d.%d.%d.%d\n", 
													lc_addr[0], lc_addr[1], lc_addr[2], lc_addr[3] );
		else
			fprintf( lh_debug_file, "Address is: NOT FOUND\n" );
		}


		{
		char *lp_ascii;


		// Read the ascii string at Test = dyn and place it in a dynamically allocated buffer
		// -big enough to take the value
        if( flml_read_ascii_dynamic( &l_parse_ctrl, 
										"Test=2dyn",
										0, &lp_ascii ) )
			{
			fprintf( lh_debug_file, "Dynamic ASCII (Dynamic) is: %s\n", lp_ascii );
			free( lp_ascii );
			}
		else
			fprintf( lh_debug_file, "Dynamic ASCII is: NOT FOUND\n" );
		}


		{
		char lc_data[7];

		// Try to read the ascii value at Test = dyn into the buffer.  This should FAIL
		// because the lc_data string is too small
        if( flml_read_ascii( &l_parse_ctrl, 
										"Test=2dyn",
										0, lc_data, sizeof(lc_data) ) )
			fprintf( lh_debug_file, "Small ASCII (should fail) is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "Small ASCII is: TOO SMALL (correct)\n" );
		}


		{
		tlms_unicode lc_unicode[10];

		// Read in the Unicode value
        if( flml_read_unicode( &l_parse_ctrl, 
										"Test=2uni",
										0, lc_unicode, sizeof(lc_unicode)/sizeof(tlms_unicode) ) )
			fprintf( lh_debug_file, "Unicode (U n i c o) is: %c %c %c %c %c\n", 
								lc_unicode[0], lc_unicode[1], lc_unicode[2], lc_unicode[3], lc_unicode[4] );
		else
			fprintf( lh_debug_file, "Unicode is: NOT FOUND\n" );
		}


		{
		char lc_embedded[1024];
		int l_embedded_length;

		// Read in the embedded value at Test = embed.  The length of the embedded string
		// -is placed in l_embedded_length
        if( flml_read_embedded( &l_parse_ctrl, 
										"Test=2embed",
										0, lc_embedded, sizeof(lc_embedded),
										&l_embedded_length ) )
			{
			lc_embedded[l_embedded_length] = '\0';
			fprintf( lh_debug_file, "Embedded ({ myco.com...) is: %s\n", 
								lc_embedded );
			}
		else
			fprintf( lh_debug_file, "Embedded is: NOT FOUND\n" );
		}


		{
		char lc_data[100];
		int l_data_length;
		char lc_reencoded_data[100];

		memset( lc_reencoded_data, 'k', sizeof(lc_reencoded_data) );

		if( flml_read_bytes( &l_parse_ctrl,
							"Test=2binary",
							0, 
							lc_data,
							sizeof( lc_data ) - 1,
							&l_data_length ) )
			{
			lc_data[l_data_length] = '\0';
			fprintf( lh_debug_file, "Binary (Aladdin:open sesame) is: %s\n", 
								lc_data );

			flml_encode_bytes( lc_reencoded_data, sizeof( lc_reencoded_data ), 
								lc_data, l_data_length );

			fprintf( lh_debug_file, "Re-encoded Binary (QWxhZGRpbjpvcGVuIHNlc2FtZQ==) is: %s\n", 
								lc_reencoded_data );
			}
		else
			fprintf( lh_debug_file, "Binary is: NOT FOUND\n" );
		}


		{
		tlms_date l_date;
		tlms_time l_time;

		if( flml_read_date( &l_parse_ctrl,
							"Test=2date",
							0, 
							&l_date ) )
			{
			fprintf( lh_debug_file, "Date (2000-04-07) is: Day:%d Month:%d Year:%d\n", 
								l_date.day, l_date.month, l_date.year );
			}
		else
			fprintf( lh_debug_file, "Date is: NOT FOUND\n" );

		if( flml_read_time( &l_parse_ctrl,
							"Test=2time",
							0, 
							&l_time ) )
			{
			fprintf( lh_debug_file, "Time (invalid format) is: %d %d %d\n", 
								l_time.hour, l_time.min, l_time.sec );
			}
		else
			fprintf( lh_debug_file, "Time is: NOT FOUND (CORRECT)\n" );

		if( flml_read_time( &l_parse_ctrl,
							"Test=2time2",
							0, 
							&l_time ) )
			{
			fprintf( lh_debug_file, "Time2 (16:40:18) is: %d %d %d\n", 
								l_time.hour, l_time.min, l_time.sec );
			}
		else
			fprintf( lh_debug_file, "Time is: NOT FOUND\n" );

		if( flml_read_time( &l_parse_ctrl,
							"Test=2time3",
							0, 
							&l_time ) )
			{
			fprintf( lh_debug_file, "Time3 (16:40) is: %d %d %d\n", 
								l_time.hour, l_time.min, l_time.sec );
			}
		else
			fprintf( lh_debug_file, "Time is: NOT FOUND\n" );
		}


		{
		tlms_oid l_oid;

		if( flml_read_oid( &l_parse_ctrl,
							"Test=2oid",
							0, 
							&l_oid ) )
			{
			unsigned int l_i;

			fprintf( lh_debug_file, "OID (0~8~245~0~7) is: " );
			for( l_i=0; l_i<l_oid.size; l_i++ )
				fprintf( lh_debug_file, "%d ", l_oid.value[l_i] );
			fprintf( lh_debug_file, "\n" );
			}
		else
			fprintf( lh_debug_file, "OID is: NOT FOUND\n" );
		}

		fprintf( lh_debug_file, "\n" );

		{
		char lc_float[256];

		// IEEE754 - Check that these conversions produce the right results 
		// -according to IEEE754.

		flml_encode_float( lc_float,     0.0 / p_zero() );
		
		fprintf( lh_debug_file, "Float (NaN) is: %s\n", lc_float );

		flml_encode_float( lc_float,     1.0 / p_zero() );
		
		fprintf( lh_debug_file, "Float (INF) is: %s\n", lc_float );

		flml_encode_float( lc_float,     -1.0 / p_zero() );
		
		fprintf( lh_debug_file, "Float (-INF) is: %s\n", lc_float );
		}


		{
		char lc_ascii_ref[] = { "Encode 'this'" };
		char lc_ascii[256];

		flml_encode_ascii( lc_ascii, sizeof( lc_ascii ), 
							lc_ascii_ref );
		
		fprintf( lh_debug_file, "Encoded ascii (Encode \\'this\\') is: %s\n",
							lc_ascii );

		fprintf( lh_debug_file, "Encoded length of ascii (15+2+1) is: %d\n",
							flml_encode_get_ascii_length( lc_ascii_ref ) );
		}


		{
		char lc_ascii_ref[] = { "Encode \"that\"" };
		tlms_unicode lc_unicode_ref[256];
		char lc_unicode[256];
		int l_i;

		for( l_i=0; lc_ascii_ref[l_i] != '\0'; l_i++ )
			lc_unicode_ref[l_i] = lc_ascii_ref[l_i];
		lc_unicode_ref[l_i] = '\0';

		flml_encode_unicode( lc_unicode, sizeof( lc_unicode ), 
							lc_unicode_ref );
		
		fprintf( lh_debug_file, "Encoded unicode (Encode \\\"that\\\") is: %s\n",
							lc_unicode );

		fprintf( lh_debug_file, "Encoded length of unicode (15+2+1) is: %d\n",
							flml_encode_get_unicode_length( lc_unicode_ref ) );
		}


		// Release the resources associated with the message parsing
        flml_release( &l_parse_ctrl );


        if( lh_debug_file != stdout )
            fclose( lh_debug_file );

        fclose( lh_fin );
        }
}


int main( int a_argc, char **app_argv )
{
    char *lp_input_file;
    char *lp_output_file;


    if( a_argc >= 2 )
        lp_input_file = app_argv[1];
    else
        lp_input_file = "test-message.txt";

    if( a_argc >= 3 )
        {
        if( app_argv[2][0] != '-' )
            lp_output_file = app_argv[2];
        else
            lp_output_file = NULL;
        }
    else
        lp_output_file = "lumas-lib-output.txt";

	f_lumas_c_test( lp_input_file, lp_output_file );

	f_lumas_cpp_test( lp_input_file, lp_output_file );


	return( 0 );
}


//------------------------------------------------------------------------------
//             Copyright (c) 2002-2003, Tech-Know-Ware Ltd.
//------------------------------------------------------------------------------
