//------------------------------------------------------------------------------
//             Copyright (c) 2002-2003, Tech-Know-Ware Ltd.
//------------------------------------------------------------------------------


#ifndef _LUMAS_LIB_H
#define _LUMAS_LIB_H


// File name:   lumas-lib.h
//------------------------------------------------------------------------------
// Description: This file contains library functions for reading Lumas encoded
//				messages.
//
// Notice:      TERMS & CONDITIONS
//
//              Your use of this file implies agreement to the Terms & 
//              Conditions as presented in this notice.
//
//              Subject to the terms and conditions of this notice, 
//              TECH-KNOW-WARE LTD hereby grants YOU (the person accepting this 
//              agreement) a non-exclusive right to use this file.
//
//              THIS FILE AND THE SOFTWARE CONTAINED HEREIN IS PROVIDED 'AS IS' 
//              AND COMES WITH NO WARRENTY OF ANY KIND.  
//
//              TECH-KNOW-WARE LTD SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED 
//              BY ANYONE OR ANYTHING DUE TO THE USE OF THIS FILE HOWEVER THEY 
//              MAY BE CAUSED.
//
//              This file, or any derivative of it, is provided on the condition
//              that YOU may not supply it to a third party outside of the 
//              organisation to which you are affiliated in a form that 
//              allows the third party to compile or link the software into 
//              programs of their own making.  Other than the previous 
//              condition, this file, or any derivative of it, may be used for 
//              any purpose, including commercial purposes. 
//
//              Users are permitted to modify the contents of this file, as 
//              long this notice and the copyright notice remains and the 
//              modified file is given a name that can not be confused with 
//              the files provided by Tech-Know-Ware Ltd.
//
//              Users are permitted to extract elements of this file into
//              other files, subject to this notice and the copyright notice 
//              also being copied into the target files.
//
//              For more information on Lumas, visit:
//
//                  http://www.tech-know-ware.com/lumas
//
//              If you find a bug in this software, please report it to:
//
//                  lumas-lib-bugs@tech-know-ware.com
//
//              Copyright (c) 2002, 2003 Tech-Know-Ware Ltd.
//
//              End of Notice.
//
// Module Tag:  lml - Lumas Library.
// Version:     V.96
//
// Changes:
// P. Cordell	2001-06-04	Preparation for evaluation
// P. Cordell	2002-03-07	V.95.4 Release
// P. Cordell	2002-04-07	Move to V.95.5
// P. Cordell	2002-04-25	Move to V.95.6
// P. Cordell	2003-01-07	V.95.7 - Rename to Lumas


#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif


#ifndef FALSE
#	define FALSE   0
#endif

#ifndef TRUE
#	define TRUE    (!FALSE)
#endif

#define KLML_MAX_TAG 64


typedef enum elml_error
{ 
	ELML_OK						= 0,
	ELML_PARSE_FAILED			= 1000,
	ELML_NO_DATA_IN_MESSAGE,
	ELML_ALLOCATION_FAILED,
	ELML_TOO_MANY_RECURSIONS,
    ELML_MESSAGE_TOO_SHORT,
    ELML_PARAMETER_TOO_LONG,
    ELML_PARAMETER_WRONG_TYPE,
    ELML_PARAMETER_NOT_FOUND,
	ELML_PARAMETER_DOES_NOT_EXIST,	// Only used when tagging info is present
	ELML_OUTPUT_TOO_SMALL
} elml_error;


typedef struct slml_tag_translation 
{
	char	*text_tag;
	int		enumerated_value;
} slml_tag_translation;


// Users may redefine this function to something more useful
#define flml_alloc( x, y ) malloc( y )


//-----------------------------------------------------------------------------
//     Internal types - Users should need to interact with these directly
//-----------------------------------------------------------------------------

typedef int tlml_input_location;

typedef struct slml_message_element
{
    char						*p_tag;
    tlml_input_location			tag_location;
	int							tag_number;
    tlml_input_location			value_location;
    struct slml_message_element	*p_compound;
    struct slml_message_element	*p_next;
} slml_message_element;


// These are for the tag info data file
extern const char glml_is_struct[];
extern const char glml_is_union[];
extern const char glml_is_untagged[];

typedef struct slml_tag_info
{
	const char *p_name;
	struct slml_tag_info *p_definition;
} slml_tag_info;


typedef struct slml_location
{
	slml_message_element	*p_element;
	slml_tag_info			*p_tag_info;
} slml_location;


typedef struct slml_parse_ctrl
{
    const char				*p_message;
	int						length;
	const char				*p_next;
	const char				*p_message_end;
	elml_error				error;
    int						allocation_failed;
	slml_location			root_location;
	slml_location			base_location;
	int						unget_char;
	int						recursion;
} slml_parse_ctrl;


typedef struct slml_encode_ctrl
{
	elml_error				error;
} slml_encode_ctrl;


//-----------------------------------------------------------------------------
//                Types for storing parameter contents
//-----------------------------------------------------------------------------

#ifndef LUMAS_TYPES
#define LUMAS_TYPES

typedef		unsigned char	tlms_bool;		// Boolean value
typedef 	unsigned long	tlms_uns32;		// Unsigned 32 bit integer
typedef 	unsigned short	tlms_uns16;		// Unsigned 16 bit integer
typedef 	unsigned char	tlms_uns8;		// Unsigned 8 bit integer
typedef 	signed long		tlms_int32;		// Signed 32 bit integer
typedef 	signed short	tlms_int16;		// Signed 16 bit integer
typedef 	signed char		tlms_int8;		// Signed 8 bit integer
typedef 	float			tlms_float;		// Single precision floating point
typedef 	double			tlms_double;	// Double precision floating point
typedef		char			tlms_ascii;		// Ascii character
typedef 	unsigned short	tlms_unicode;	// 16 bit wide Unicode character
typedef 	unsigned char	tlms_bytes;		// Bytes type
typedef		char			tlms_embedded;	// Single byte of an embedded type

typedef tlms_uns8 tlms_ipv4[4];

typedef tlms_uns8 tlms_ipv6[16];

typedef struct 
{
	unsigned int year;	/* years since 0 */
	unsigned int month;	/* months since January - [1,12] */
	unsigned int day;	/* day of the month - [1,31] */
} tlms_date;

typedef struct 
{
	unsigned int hour;    /* hours since midnight - [0,23] */
	unsigned int min;     /* minutes after the hour - [0,59] */
	unsigned int sec;     /* seconds after the minute - [0,59] */
} tlms_time;

typedef struct
{
	unsigned int size;
	unsigned int value[20];
} tlms_oid;

#endif // LUMAS_TYPES


//-----------------------------------------------------------------------------
//                   Functions for setting up parsing
//-----------------------------------------------------------------------------


int flml_is_message_complete( const char *ap_message, int a_length );

void flml_set_tag_info_root( slml_parse_ctrl *ap_parse_ctrl, slml_tag_info *ap_tag_info );

tlms_bool flml_parse_message( slml_parse_ctrl *ap_parse_ctrl, 
								const char *ap_message, 
								int a_length );

elml_error flml_get_last_error( slml_parse_ctrl *ap_parse_ctrl );


//-----------------------------------------------------------------------------
//          Functions for debugging and seeing what is going on
//-----------------------------------------------------------------------------


void flml_debug_display_parse_info( FILE *ah_fout, slml_parse_ctrl *ap_parse_ctrl );


//-----------------------------------------------------------------------------
//          Functions for changing the base location in the message
//-----------------------------------------------------------------------------

tlms_bool flml_set_location( slml_parse_ctrl *ap_parse_ctrl,
								const char ac_tagging[], 
								int a_instance,
								slml_location *ap_old_location );

slml_location flml_restore_location( slml_parse_ctrl *ap_parse_ctrl,
										slml_location a_base );


//-----------------------------------------------------------------------------
//                       Functions for reading parameters
//-----------------------------------------------------------------------------

tlms_bool flml_is_present( slml_parse_ctrl *ap_parse_ctrl, 
									const char ac_tagging[],
									int a_instance );

tlms_bool flml_read_void( slml_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance );

tlms_bool flml_read_boolean( slml_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									int *ap_boolean );

tlms_bool flml_read_integer( slml_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									int *ap_integer );

tlms_bool flml_read_float( slml_parse_ctrl *ap_control,
									const char ac_tagging[],
									int a_instance, 
									tlms_float *ap_float );

tlms_bool flml_read_double( slml_parse_ctrl *ap_control,
									const char ac_tagging[],
									int a_instance, 
									tlms_double *ap_double );

tlms_bool flml_read_ipv4( slml_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									unsigned char ac_addr[4] );

tlms_bool flml_read_ipv6( slml_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									unsigned char ac_addr[16] );

tlms_bool flml_read_date( slml_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									tlms_date *ap_date );

tlms_bool flml_read_time( slml_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									tlms_time *ap_time );

tlms_bool flml_read_oid( slml_parse_ctrl *ap_parse_ctrl,
								const char ac_tagging[],
								int a_instance, 
								tlms_oid *ap_oid );

tlms_bool flml_read_ascii( slml_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									char ac_string[],
									int a_max_length );

tlms_bool flml_read_ascii_dynamic( slml_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									char **app_string );

tlms_bool flml_read_const( slml_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									char ac_string[],
									int a_max_length );

tlms_bool flml_read_unicode( slml_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									tlms_unicode ac_string[],
									int a_max_length );

tlms_bool flml_read_unicode_dynamic( slml_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									tlms_unicode **app_string );

tlms_bool flml_read_bytes( slml_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									char ac_string[],
									int a_max_length,
									int *ap_result_length );

tlms_bool flml_read_bytes_dynamic( slml_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									char **app_data,
									int *ap_result_length );

tlms_bool flml_read_embedded( slml_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									char ac_string[],
									int a_max_length,
									int *ap_result_length );

tlms_bool flml_read_embedded_dynamic( slml_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									char **app_message,
									int *ap_result_length );

tlms_bool flml_read_union( slml_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									char ac_string[],
									int a_max_length );

int flml_read_enumerated_union( slml_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									slml_tag_translation ac_tag_translation[] );


//-----------------------------------------------------------------------------
//                      Functions for clearing up
//-----------------------------------------------------------------------------

void flml_release( slml_parse_ctrl *ap_parse_ctrl );


//-----------------------------------------------------------------------------
//                      Functions to support encoding
//-----------------------------------------------------------------------------

tlms_bool flml_encode_float( char ac_encoded_float[], 
								tlms_double a_value );

int flml_encode_get_ascii_length( const char ac_ascii[] );

tlms_bool flml_encode_ascii( char ac_encoded_ascii[], 
									int a_encoded_ascii_max_length,
									const char ac_ascii[] );

int flml_encode_get_unicode_length( const tlms_unicode ac_unicode[] );

tlms_bool flml_encode_unicode( char ac_encoded_unicode[], 
									int a_encoded_unicode_max_length,
									const tlms_unicode ac_unicode[] );

int flml_encode_get_bytes_length( int a_data_length );

tlms_bool flml_encode_bytes( char ac_base64[],
									int a_max_length,
									const char *ap_data,
									int a_length );

tlms_bool flml_is_well_formed(  const char *ap_message, int a_length );


#ifdef __cplusplus
	}
#endif


#endif


//------------------------------------------------------------------------------
//             Copyright (c) 2002-2003, Tech-Know-Ware Ltd.
//------------------------------------------------------------------------------
