//------------------------------------------------------------------------------
//             Copyright (c) 2002-2003, Tech-Know-Ware Ltd.
//------------------------------------------------------------------------------


#ifndef _LUMAS_LIB_HPP
#define _LUMAS_LIB_HPP


// File name:   lumas-lib.h
//------------------------------------------------------------------------------
// Description: This file contains library functions for reading Lumas encoded
//				messages.
//
// Notice:      TERMS & CONDITIONS
//
//              Your use of this file implies agreement to the Terms & 
//              Conditions as presented in this notice.
//
//              Subject to the terms and conditions of this notice, 
//              TECH-KNOW-WARE LTD hereby grants YOU (the person accepting this 
//              agreement) a non-exclusive right to use this file.
//
//              THIS FILE AND THE SOFTWARE CONTAINED HEREIN IS PROVIDED 'AS IS' 
//              AND COMES WITH NO WARRENTY OF ANY KIND.  
//
//              TECH-KNOW-WARE LTD SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED 
//              BY ANYONE OR ANYTHING DUE TO THE USE OF THIS FILE HOWEVER THEY 
//              MAY BE CAUSED.
//
//              This file, or any derivative of it, is provided on the condition
//              that YOU may not supply it to a third party outside of the 
//              organisation to which you are affiliated in a form that 
//              allows the third party to compile or link the software into 
//              programs of their own making.  Other than the previous 
//              condition, this file, or any derivative of it, may be used for 
//              any purpose, including commercial purposes. 
//
//              Users are permitted to modify the contents of this file, as 
//              long this notice and the copyright notice remains and the 
//              modified file is given a name that can not be confused with 
//              the files provided by Tech-Know-Ware Ltd.
//
//              Users are permitted to extract elements of this file into
//              other files, subject to this notice and the copyright notice 
//              also being copied into the target files.
//
//              For more information on Lumas, visit:
//
//                  http://www.tech-know-ware.com/lumas
//
//              If you find a bug in this software, please report it to:
//
//                  lumas-lib-bugs@tech-know-ware.com
//
//              Copyright (c) 2002, 2003 Tech-Know-Ware Ltd.
//
//              End of Notice.
//
// Module Tag:  lml - Lumas Library.
// Version:     V.96
//
// Changes:
// P. Cordell	2001-06-17	Preparation for evaluation
// P. Cordell	2002-03-07	V.95.4 Release
// P. Cordell	2002-04-07	Move to V.95.5
// P. Cordell	2002-04-25	Move to V.95.6
// P. Cordell	2003-01-07	V.95.7 - Rename to Lumas


#include "lumas-lib.h"

extern "C" tlms_bool flml_initialise( slml_parse_ctrl *ap_aprse_ctrl );


class c_lumas
{
private:
	slml_parse_ctrl		o_parse_ctrl;

public:
	c_lumas()
		{
		flml_initialise( &o_parse_ctrl );
		}

	c_lumas( char *ap_message, int a_length )  
		{ 
		flml_parse_message( &o_parse_ctrl, ap_message, a_length ); 
		}


	static int f_is_message_complete( char *ap_message, int a_length )
		{
		return flml_is_message_complete( ap_message, a_length );
		}

	void f_set_tag_info_root( slml_tag_info *ap_tag_info )
		{
		flml_set_tag_info_root( &o_parse_ctrl, ap_tag_info );
		}

	tlms_bool f_parse_message( char *ap_message, int a_length )
		{
		return flml_parse_message( &o_parse_ctrl, ap_message, a_length );
		}


	elml_error f_get_last_error()
		{
		return flml_get_last_error( &o_parse_ctrl );
		}


	//-----------------------------------------------------------------------------
	//          Functions for debugging and seeing what is going on
	//-----------------------------------------------------------------------------


	void f_debug_display_parse_info( FILE *ah_fout )
		{
		flml_debug_display_parse_info( ah_fout, &o_parse_ctrl );
		}


	//-----------------------------------------------------------------------------
	//          Functions for changing the base location in the message
	//-----------------------------------------------------------------------------

	tlms_bool f_set_location( char ac_tagging[], int a_instance )
		{
		return flml_set_location( &o_parse_ctrl, ac_tagging, a_instance, NULL );
		}


	tlms_bool f_set_location( slml_location *ap_old_location, 
											char ac_tagging[], 
											int a_instance )
		{
		return flml_set_location( &o_parse_ctrl, ac_tagging, a_instance, ap_old_location );
		}


	slml_location f_restore_location( slml_location a_old_location )
		{
		return flml_restore_location( &o_parse_ctrl, a_old_location );
		}


	//-----------------------------------------------------------------------------
	//                       Functions for reading parameters
	//-----------------------------------------------------------------------------

	tlms_bool f_is_present( const char ac_tagging[], 
								int a_instance )
		{
		return flml_is_present( &o_parse_ctrl, ac_tagging, a_instance );
		}


	tlms_bool f_read_void( const char ac_tagging[], 
								int a_instance )
		{
		return flml_read_void( &o_parse_ctrl, ac_tagging, a_instance );
		}


	tlms_bool f_read_boolean( const char ac_tagging[], 
								int a_instance, 
								int *ap_boolean )
		{
		return flml_read_boolean( &o_parse_ctrl, ac_tagging, a_instance, ap_boolean );
		}


	tlms_bool f_read_integer( const char ac_tagging[], 
								int a_instance, 
								int *ap_integer )
		{
		return flml_read_integer( &o_parse_ctrl, ac_tagging, a_instance, ap_integer );
		}


	tlms_bool f_read_float( const char ac_tagging[],
								int a_instance, 
								tlms_float *ap_float )
		{
		return flml_read_float( &o_parse_ctrl, ac_tagging, a_instance, ap_float );
		}

	tlms_bool f_read_double( const char ac_tagging[],
								int a_instance, 
								tlms_double *ap_double )
		{
		return flml_read_double( &o_parse_ctrl, ac_tagging, a_instance, ap_double );
		}

	tlms_bool f_read_ipv4( const char ac_tagging[], 
								int a_instance, 
								unsigned char ac_addr[4] )
		{
		return flml_read_ipv4( &o_parse_ctrl, ac_tagging, a_instance, ac_addr );
		}


	tlms_bool f_read_ipv6( const char ac_tagging[],
									int a_instance, 
									unsigned char ac_addr[16] )
		{
		return flml_read_ipv6( &o_parse_ctrl, ac_tagging, a_instance, ac_addr );
		}


	tlms_bool f_read_date( const char ac_tagging[],
									int a_instance, 
									tlms_date *ap_date )
		{
		return flml_read_date( &o_parse_ctrl, ac_tagging, a_instance, ap_date );
		}


	tlms_bool f_read_time( const char ac_tagging[],
									int a_instance, 
									tlms_time *ap_time )
		{
		return flml_read_time( &o_parse_ctrl, ac_tagging, a_instance, ap_time );
		}


	tlms_bool f_read_oid( const char ac_tagging[],
									int a_instance, 
									tlms_oid *ap_oid )
		{
		return flml_read_oid( &o_parse_ctrl, ac_tagging, a_instance, ap_oid );
		}


	tlms_bool f_read_ascii( const char ac_tagging[],
									int a_instance, 
									char ac_string[],
									int a_max_length )
		{
		return flml_read_ascii( &o_parse_ctrl,
									ac_tagging,
									a_instance, 
									ac_string,
									a_max_length );
		}


	tlms_bool f_read_ascii_dynamic( const char ac_tagging[],
									int a_instance, 
									char **app_string )
		{
		return flml_read_ascii_dynamic( &o_parse_ctrl,
										ac_tagging,
										a_instance, 
										app_string );
		}


	tlms_bool f_read_const( const char ac_tagging[],
									int a_instance, 
									char ac_string[],
									int a_max_length )
		{
		return flml_read_const( &o_parse_ctrl,
									ac_tagging,
									a_instance, 
									ac_string,
									a_max_length );
		}


	tlms_bool f_read_unicode( const char ac_tagging[],
									int a_instance, 
									tlms_unicode ac_string[],
									int a_max_length )
		{
		return flml_read_unicode( &o_parse_ctrl,
									ac_tagging,
									a_instance, 
									ac_string,
									a_max_length );
		}


	tlms_bool f_read_unicode_dynamic( const char ac_tagging[],
										int a_instance, 
										tlms_unicode **app_string )
		{
		return flml_read_unicode_dynamic( &o_parse_ctrl,
											ac_tagging,
											a_instance, 
											app_string );
		}


	tlms_bool f_read_bytes( const char ac_tagging[],
									int a_instance, 
									char ac_string[],
									int a_max_length,
									int *ap_result_length )
		{
		return flml_read_bytes( &o_parse_ctrl,
										ac_tagging,
										a_instance, 
										ac_string,
										a_max_length,
										ap_result_length );
		}


	tlms_bool f_read_bytes_dynamic( const char ac_tagging[],
									int a_instance, 
									char **app_data,
									int *ap_length )
		{
		return flml_read_bytes_dynamic( &o_parse_ctrl,
												ac_tagging,
												a_instance, 
												app_data,
												ap_length );
		}


	tlms_bool f_read_embedded( const char ac_tagging[],
									int a_instance, 
									char ac_string[],
									int a_max_length,
									int *ap_result_length )
		{
		return flml_read_embedded( &o_parse_ctrl,
									ac_tagging,
									a_instance, 
									ac_string,
									a_max_length,
									ap_result_length );
		}


	tlms_bool f_read_embedded_dynamic( const char ac_tagging[],
										int a_instance, 
										char **app_message,
										int *ap_length )
		{
		return flml_read_embedded_dynamic( &o_parse_ctrl,
											ac_tagging,
											a_instance, 
											app_message,
											ap_length );
		}


	tlms_bool f_read_union( const char ac_tagging[],
									int a_instance, 
									char ac_string[],
									int a_max_length )
		{
		return flml_read_union( &o_parse_ctrl,
									ac_tagging,
									a_instance, 
									ac_string,
									a_max_length );
		}


	int f_read_enumerated_union( const char ac_tagging[],
									int a_instance, 
									slml_tag_translation ac_tag_translation[] )
		{
		return flml_read_enumerated_union( &o_parse_ctrl,
											ac_tagging,
											a_instance, 
											ac_tag_translation );
		}


	//-----------------------------------------------------------------------------
	//                      Functions for clearing up
	//-----------------------------------------------------------------------------

	
	void f_release()
		{
		flml_release( &o_parse_ctrl );
		}


	~c_lumas()
		{
		flml_release( &o_parse_ctrl );
		}
};


#endif


//------------------------------------------------------------------------------
//             Copyright (c) 2002-2003, Tech-Know-Ware Ltd.
//------------------------------------------------------------------------------
